import os
from functools import partial
import maya.cmds as cmds
import hmScripts as root
import hmScripts.hm_global_variables as global_vars
from hmScripts.hm_global_variables import language

import hmScripts.Main.hmNodeConnector as hmConn
import hmScripts.Main.hmNetworkHandler as hnNetHand
import hmScripts.Main.hmBaseObject as hmNode

import hmScripts.UI.HmOptions_HardMeshOperation as oper_opt
import hmScripts.UI.HmOptions_OperatorsStack as oper_stack
import hmScripts.UI.HmMenu_OperatorEdit as edit_menu

import hmScripts.UI.HmWindow_BaseOption as base_win
import hmScripts.Main.hmUtility as util

import hmScripts.Main.hmPerform_new_operation as perf

rootPath = root.__path__ [0]
mayaVersion = int( str(cmds.about(api=True))[0:4] )
labels = language.values['workflow_win']
titles = language.values['win_titles']
help_texts = language.values['help_win']

# -------------------------------------------------------------------------------------------------------------------- #
class HardMeshDockUI(object):
    """
    This window put together all the sub elements for a complete workflow window that reacts
    to the scene selection and updates the attributes and the stack from the operator stack function.
    Ideally this will go away and be sobstituted by different windows that can pe attached together and
    communicate through underlying signals.
    """
    hmNAME = 'hardMeshWorkspaceWindow'
    hmTITLE = titles['workflow_win']
    currentInstance = None
    width = 410

    # Internal options
    VERBOSE = 0

    def __init__(self):

        if HardMeshDockUI.currentInstance is not None:
            del HardMeshDockUI.currentInstance

        HardMeshDockUI.currentInstance = self

        self.main_parent = None
        self.options_frame = None
        self.job_num = None
        self.tooltips = labels['tooltips']

        # ------------------------------------------------------------------------------------------------------------ #
        # Initialize the instance of the network handler
        self.network_handler = hnNetHand.HmNetworkHandler()

        # Initialize the attribute section for operation creation and editing
        self.hard_mesh_edit_content = oper_opt.HardMeshOperationOptionsUI(as_parameter_changer=True)

        # Window classes initialization
        self.hard_mesh_op_content = oper_opt.HardMeshOperationOptionsUI()
        self.hard_mesh_op_window = base_win.HmOptionWindow('Hard_Mesh_Operation', self.hard_mesh_op_content)

        # Initialize the operators stack with the current Handler and the attributes changer
        self.operators_stack = oper_stack.OperatorsStack(self.network_handler,
                                                         attribute_object=self.hard_mesh_edit_content)

        # Initialize the edit menu
        self.edit_menu = edit_menu.OperatorMenu(self.network_handler, self.operators_stack)

        #Links
        self.pux_site = 'http://www.pux-3d.com'
        docfile = os.path.join(rootPath, 'UI', 'doc_link.txt')
        with open(docfile) as f:
            self.current_dok_link = f.readlines()[0]

    # ---------------------------------------------------------------------------------------------------------------- #
    @staticmethod
    def get_instance():
        """
        returns instance of a currently opened main window; returned value is only valid while window is opened.
        :rtype: HardMeshDockUI
        """

        if HardMeshDockUI.currentInstance is None:
            HardMeshDockUI()

        return HardMeshDockUI.currentInstance

    @staticmethod
    def open(*args):

        if mayaVersion >= 2017:
            # Delete if exist
            if cmds.workspaceControl(HardMeshDockUI.hmNAME, exists=True):
                cmds.deleteUI( HardMeshDockUI.hmNAME )

            # Make a new workspaceControl
            cmds.workspaceControl(	HardMeshDockUI.hmNAME,
                                    label = HardMeshDockUI.hmTITLE,
                                    uiScript ='import hmScripts.UI.HmWindow_Workflow as UI; '
                                              'UI.HardMeshDockUI().resume_in_workspace_control()',
                                    dockToControl = ['MainPane', 'left'],
                                    initialHeight = 400,
                                    initialWidth = HardMeshDockUI.width,
                                    )

        else:
            if cmds.dockControl(HardMeshDockUI.hmNAME, exists=True):
                cmds.deleteUI( HardMeshDockUI.hmNAME )

            hmWin = cmds.window( menuBar=True )
            layout = cmds.frameLayout(labelVisible = False)
            HardMeshDockUI().create_window_content(layout)

            # Make window a dock control
            cmds.dockControl(HardMeshDockUI.hmNAME, label= HardMeshDockUI.hmTITLE , area='left',
                             width = HardMeshDockUI.width, allowedArea = ['left', 'right'], content= hmWin)

    @classmethod
    def resume_in_workspace_control(cls):
        """
        used by workspace (Maya 2017+) to restore the main window control
        """
        try:
            window = HardMeshDockUI.get_instance()
            window.create_window_content(cmds.setParent(q=True))
        except Exception as err:
            print err

    # ---------------------------------------------------------------------------------------------------------------- #
    def create_window_content(self, parent):
        """
        This method put together all the elements for the UI creation
        """
        # --------------------------------------- ATTRIBUTES UPDATE -------------------------------------------------- #
        #create a script job that will update the interface when a module is selected.
        self.job_num = cmds.scriptJob(e= ["SelectionChanged", self.update_hm_UI_from_selection])

        # Temporary script job for the killing of the main one
        cmds.scriptJob(runOnce = True, uiDeleted=(parent, self.kill_script_job))


        # ---------------------------------------------- UI ELEMENTS-------------------------------------------------- #
        self.main_parent = parent
        cmds.setParent(self.main_parent)

        # Menu Bar and Creation buttons
        self.main_menu()
        self.creation_buttons()

        # Pane Layout for resizing the widgets and Hard Mesh Operators Stack
        cmds.paneLayout( configuration='horizontal2', paneSize = [1, 100, 30] )
        self.operators_stack.create_ui_elements()

        # Hard Mesh Parameters
        cmds.scrollLayout(cr=True)
        self.options_frame = cmds.frameLayout(label ='Basic Parameters',
                                              collapsable= False,
                                              collapse = False,
                                              enable = False)
        self.hard_mesh_edit_content.createUiElements()

        # Set the parent, for the next controls, to the mayn column layout
        cmds.setParent( '..' )

    def main_menu(self):
        """
        The menu bar has all the commands that should be found on a menu..
        """
        cmds.menuBarLayout()

        # Edit menu and commands
        cmds.menu( label=labels['edit_menu']['title'], tearOff = True)
        self.edit_menu.create_ui_elements()

        # Utility menu and commands
        util_menu_lb = labels['util_menu']

        cmds.menu( label=util_menu_lb['title'], tearOff = True)
        cmds.menuItem( divider = True, label=util_menu_lb['in_out'])
        cmds.menuItem( label=util_menu_lb['clean_dup_in'], c= lambda *_:util.clean_duplicate_input())
        cmds.menuItem( label=util_menu_lb['clean_dup_out'], c= lambda *_:util.clean_duplicate_output())
        cmds.menuItem(label=util_menu_lb['ref_copy'], c= lambda *_:util.perform_reference_copy())

        cmds.menuItem( divider = True, label=util_menu_lb['cleaning'])
        keep_in_meshes = cmds.menuItem( label=util_menu_lb['keep_in_mesh'], checkBox=True )
        cmds.menuItem( label=util_menu_lb['clean_curr'],
                       ann = self.tooltips['clean_curr'],
                       c=lambda *_:self.network_handler.clean_current( cmds.menuItem(keep_in_meshes,
                                                                                     q=True,
                                                                                     checkBox = True ) ) )

        cmds.menuItem( label=util_menu_lb['clean_scene'],
                       ann = self.tooltips['clean_scene'],
                       c=lambda *_:self.network_handler.clean_scene( cmds.menuItem(keep_in_meshes,
                                                                                   q=True,
                                                                                   checkBox = True ) ) )
        # Option menu and command
        options_lb = labels['options_menu']

        cmds.menu( label=options_lb['title'], tearOff = True)
        cmds.menuItem(divider=True, dividerLabel = 'Editing')
        cmds.menuItem( label=options_lb['auto_lock'],
                       ann = self.tooltips['auto_lock'],
                       cb=True,
                       c = self.set_auto_lock_option )

        # Help menu and commands
        help_lb = labels['help_menu']

        cmds.menu( label=help_lb['title'], helpMenu=True, tearOff = True )
        cmds.menuItem( label=help_lb['hm_doc'], c= self.open_html_help )
        cmds.menuItem( label=help_lb['about'], c= self.open_html_about )
        cmds.menuItem( label=help_lb['info'], c= self.open_info_win )

    def creation_buttons(self):
        """
        This set of button contains all the operations that can be performed with an hmCompound network
        """
        cmds.columnLayout(adjustableColumn = True)


        cmds.rowLayout( numberOfColumns=9, height=50 )

        cmds.shelfButton(image1="hm2_opUnion.png",
                         ann = self.tooltips['op_union']+self.tooltips['double_click'],
                         c = partial(self.executeOperationAs, 1),
                         dcc = partial(self.openWindowsAs, 1)
                         )

        cmds.shelfButton(image1="hm2_opDiffAB.png", ndp=True,
                         ann = self.tooltips['op_diff_ab']+self.tooltips['double_click'],
                          c = partial(self.executeOperationAs, 2),
                         dcc = partial(self.openWindowsAs, 2)
                         )

        cmds.shelfButton(image1="hm2_opDiffBA.png",  ndp=True,
                         ann = self.tooltips['op_diff_ba']+self.tooltips['double_click'],
                         c = partial(self.executeOperationAs, 3),
                         dcc = partial(self.openWindowsAs, 3)
                         )

        cmds.shelfButton(image1="hm2_opInter.png",  ndp=True,
                         ann = self.tooltips['op_inter']+self.tooltips['double_click'],
                         c = partial(self.executeOperationAs, 4),
                         dcc = partial(self.openWindowsAs, 4)
                         )

        cmds.shelfButton(image1="hm2_opPanAB.png", ndp=True,
                         ann = self.tooltips['op_panel_ab']+self.tooltips['double_click'],
                         c = partial(self.executeOperationAs, 5),
                         dcc = partial(self.openWindowsAs, 5)
                         )

        cmds.shelfButton(image1="hm2_opPanBA.png",  ndp=True,
                         ann = self.tooltips['op_panel_ba']+self.tooltips['double_click'],
                         c = partial(self.executeOperationAs, 6),
                         dcc = partial(self.openWindowsAs, 6)
                         )

        if global_vars.RELEASE >= 2.5:
            # == Separator
            cmds.separator(horizontal = False, style = 'single', w = 20, h = 50, vis = True)

            # == Additional, future commands
            cmds.shelfButton(image1="hm2_split_mesh_setup.png",
                             ndp=True,
                             c = partial(perf.perform_split_multi_operation),
                             dcc = 'print "No options yet"',
                             label = '',
                             ann = '' )

            # cmds.shelfButton(image1="pencil.png", ndp=True, c = 'print 1', dcc = 'print 1', label = 'ON M', ann = 'Mesh from Curve') # ann = self.tooltipArray['Visibility'],

        #Set parent back to row
        cmds.setParent( '..' )

        # Add the separator
        cmds.separator( height=10, style='in' )

        #Set parent back to column
        cmds.setParent( '..' )

    # ---------------------------------------------------------------------------------------------------------------- #
    @util.undo
    def update_hm_UI_from_selection(self, *args):
        """Update the interface if something interesting is selected."""

        def activate_elements():
            # Activate the elements
            self.operators_stack.activate()
            cmds.frameLayout(self.options_frame, edit=True, enable=True)

        def deactivate_elements():
            # Deactivate the elements
            self.operators_stack.deactivate()
            cmds.frameLayout(self.options_frame, edit=True, enable=False)

        def in_component_mode():
            # Query if we are in global component selection
            if cmds.selectMode(q=True, component=True):
                return True
            # If we are in local component selection..
            elif cmds.selectType(q=True, ocm=True, alc=True):
                return True
            else:
                return False

        # ------------------------------------------------------------------------------------------------------------ #
        def update_it():
            """The actual function that does the job"""
            deactivate = True

            # Get the selection
            sel = cmds.ls(sl=True, long=True)

            # Print message
            if self.VERBOSE: print 'Selection is %s ' % sel

            # # If we have a selection
            if len(sel) > 0:
                current_object = hmNode.HmBaseObject(sel[0])
                curr_obj_name = current_object.get_name()

                # ------------------------------------ COMPONENT SELECTION CASE -------------------------------------- #
                # In case we are in component mode get the transform for doing the query
                if in_component_mode():
                    # we do it twice because the component count as an extra shape
                    transform = current_object.get_transform().get_transform()
                    if transform:
                        # print transform
                        curr_obj_name = transform.get_name()
                        # print curr_obj_name

                # Query if the object type is relevant
                obj_type = hmConn.HmOperationConnect.get_object_type(curr_obj_name)

                # Print message
                if self.VERBOSE: print 'The selected object %s is of type %s ' % (curr_obj_name, obj_type)

                # -------------------------------------- IF RELEVANT OBJECT ------------------------------------------ #
                if obj_type:
                    # worth_updating = True

                    # Make sure that the UI elements are active
                    activate_elements()

                    # ---------------------------------------------- UPDATE ------------------------------------------ #
                    # if worth_updating:
                    deactivate = False

                    # Initialize the Handler with the current valid object
                    self.network_handler.initialize_from_selection(curr_obj_name, obj_type)

                    # Set the UI updates in the operators stack
                    self.operators_stack.update()

                    # Print message
                    if self.VERBOSE: print 'Handler and UI updated.'

            if deactivate:
                # Deactivate UI
                deactivate_elements()
                # Print message
                if self.VERBOSE: print 'UI deactivated nothing selected.'

        # ------------------------------------------------------------------------------------------------------------ #
        # print 'curr_networks after all the cheks', self.network_handler.net_objs
        # If the window is currently visible
        if mayaVersion >= 2017:
            if cmds.workspaceControl( self.hmNAME, q=True, visible = True):
                update_it()
        else:
            if cmds.dockControl(self.hmNAME, q=True, visible = True):
                update_it()

    def kill_script_job(self):
        """Kill the attribute update script job"""
        cmds.scriptJob(kill=self.job_num, force=True)

    # ---------------------------------------------------------------------------------------------------------------- #
    def open_html_help(self, *Args ):
        os.startfile(self.current_dok_link)

    def open_html_about(self, *Args ):
        os.startfile(self.pux_site)

    @staticmethod
    def open_info_win(*args):
        """
        display info on plugin
        """
        winName = 'infoWin'
        version = cmds.pluginInfo('hmTools', query=True, v=True )
        ui_version = global_vars.RELEASE

        if cmds.window(winName, exists=True):
            cmds.deleteUI( winName )

        window = cmds.window( winName, title=help_texts['title'],  widthHeight=(280, 135), sizeable=False )
        cmds.columnLayout( adjustableColumn=True )
        cmds.separator( width = 30, height=20, style='in' )
        cmds.text( label='{}\n\n{} {}\n{} {}\n\n{}'.format(help_texts['hm_vers'],
                                                           help_texts['core'],
                                                           version,
                                                           help_texts['ui'],
                                                           ui_version,
                                                           help_texts['pux'],
                                                           ),
                   align='center' )
        cmds.separator( width = 30, height=20, style='in' )
        cmds.setParent( '..' )
        cmds.showWindow( window )

    def openWindowsAs(self, val ):
        self.hard_mesh_op_content.setForcedAttributes(val)
        self.hard_mesh_op_window.buildAndShowWindow()

    def executeOperationAs(self, val):
        self.hard_mesh_op_content.setForcedAttributes(val)
        self.hard_mesh_op_window.performFromMenu()

    def set_auto_lock_option(self, *values):
        """Set the operator stack option for the auto lock option"""
        # Set the option of the operator stack
        self.operators_stack.AUTO_LOCK_UNLOCK_RESULT = values[0]

    def set_children_display(self, *values):
        self.operators_stack.ITEMS_INCLUDES_HIERARCHY = values[0]
        self.operators_stack.update()

