#!/usr/bin/env python
# -*- coding: utf-8 -*-

import maya.cmds as cmds

# Naming conventions
import hmScripts.hm_global_variables as nc


'''
This class implements a basic object for querying, creating and deleting connections and attributes
'''

class HmBaseObject(object):

    # Define the valid type of objects that we can handle
    OBJECT_TYPES = (nc.hm_in_mesh, nc.hm_out_mesh, nc.hm_inter_curve)

    # Attributes
    VERBOSE = 0


    def __init__(self, input_obj):
        '''
        Initialize the class.

        Args:
            input: a string, rapresenting a maya object used in an hm network

        Returns:
            HmBaseObject
        '''

        # Basic attributes
        # self.name = self.__validate_name(input_obj)
        self.name = input_obj

        if self.VERBOSE: print('New HmBaseObject initializated with the name of: %s' % self.name)

    def __repr__(self):

        return 'HmObj: %s' % self.get_name()


    def __str__(self):
        return self.get_name()


    def delete(self):
        '''
        Delete itself from the maya scene
        '''
        # print 'delete', self.name

        # Avoid errors if the object does not exist
        try:
            cmds.delete(self.name)
        except:
            pass
    # ================================================================================================================ #
    #================================================ NAME METHODS =================================================== #
    # ================================================================================================================ #
    def set_name(self, in_name):
        self.name = cmds.rename( self.name, in_name )
        return self

    def get_name(self):
        return self.name


    # ================================================================================================================ #
    # ============================================= TRANSFORMATION METHODS =========================================== #
    # ================================================================================================================ #
    @property
    def parent(self):
        """
        :return: The parent transform of this transform. None if no parent.
        """
        parent = cmds.listRelatives(self.name, parent=True)
        return self.__class__(parent[0]) if parent else None

    @parent.setter
    def parent(self, new_parent=None):
        """
        Move the current transform under a new transform.

        :param new_parent: name of the new parent transform.
        :return: None
        """
        if new_parent is None and self.parent:
            cmds.parent(self.name, w=True)
        else:
            cmds.parent(self.name, new_parent)

    @property
    def translate(self):
        return cmds.getAttr('{}'.format(self.name, 'translate'))

    @translate.setter
    def translate(self, translate):
        cmds.setAttr('{}.t'.format(self.name), *translate)

    @property
    def world_translate(self):
        return cmds.xform(self.name, q=True, t=True, ws=True)

    @world_translate.setter
    def world_translate(self, translate):
        cmds.xform(self.name, t=translate, ws=True)

    @property
    def rotate(self):
        return cmds.getAttr('{}'.format(self.name, 'rotate'))

    @rotate.setter
    def rotate(self, rotate):
        cmds.xform(self.name, ro=rotate, os=True)

    @property
    def world_rotate(self):
        return cmds.xform(self.name, q=True, ro=True, ws=True)

    @world_rotate.setter
    def world_rotate(self, rotate):
        cmds.xform(self.name, ro=rotate, ws=True)

    @property
    def scale(self):
        return cmds.getAttr('{}'.format(self.name, 'scale'))

    @scale.setter
    def scale(self, scale):
        cmds.xform(self.name, s=scale, os=True)

    @property
    def world_scale(self):
        return cmds.xform(self.name, q=True, s=True, ws=True)

    @world_scale.setter
    def world_scale(self, scale):
        cmds.xform(self.name, s=scale, ws=True)

    @property
    def matrix(self):
        return cmds.xform(self.name, q=True, m=True, os=True)

    @matrix.setter
    def matrix(self, matrix):
        cmds.xform(self.name, m=matrix, os=True)

    @property
    def world_matrix(self):
        return cmds.xform(self.name, q=True, m=True, ws=True)

    @world_matrix.setter
    def world_matrix(self, matrix):
        cmds.xform(self.name, m=matrix, ws=True)

    def add_group(self, suffix='GRP'):
        """
        Add a group on top
        :param suffix: The suffix to use on the group
        :return HmBaseObject: instance of the created group
        """
        grp = HmBaseObject(cmds.createNode('transform', n='{}_{}'.format(self.name, suffix)))
        grp.world_matrix = self.world_matrix
        self.parent = grp

        return grp

    # ================================================================================================================ #
    #=============================================== CONNECTIONS METHODS =============================================#
    # ================================================================================================================ #
    def is_connected(self, in_attribute):
        '''
        Query if the specified attribute is connected with the other attribute

        Args:
            in_attribute: The attribute to query

        Returns:
            Bool: True is has connection, False if not
        '''
        NotImplemented

    def get_connected(self, in_attribute, conn_type, node_type = None):
        '''
        Get the connected node to the specified attribute

        Args:
            in_attribute: The attribute to query
            conn_type: The side of the query. Valid Types are 'in' or 'out'

        Returns:
            list of tuple[(HmBaseObject, string connected attribute), ... ]
            or None if no connection is found

        '''

        # Get the connected node and attribute name
        if conn_type == 'out':
            # print 'verifico out di', self.name
            node_attribute = cmds.listConnections('{}.{}'.format(self.name, in_attribute), s=False, d=True, p= True)
        if conn_type == 'in':
            # print 'verifico in di', self.name
            node_attribute = cmds.listConnections('{}.{}'.format(self.name, in_attribute), s=True, d=False, p= True)

        # Format the value for the return
        if node_attribute:
            # Print message
            if self.VERBOSE: print('Found this connections %s ' % node_attribute)

            # A return list with multiple tuples inside
            return_list = []

            for connection in node_attribute:

                # Split the string at every point
                splitted = connection.split('.')

                # If we obtain 2 elements is a regular attribute
                if len(splitted) == 2:
                    name = splitted[0]
                    attribute = splitted[1]

                # If we obtain more than 2 elements is an array attribute and we merge it
                if len(splitted) > 2:
                    name = splitted[0]
                    attribute = '.'.join(splitted[1:])


                # Initialize the connected node as hmNode
                connectedNode = HmBaseObject(name)
                if self.VERBOSE: print('New hmNode created from %s for the output og get_connected' % connectedNode.get_name())


                # If we are searching for an exact node type
                if node_type != None:
                    if connectedNode.is_type(node_type):
                        return_list.append((connectedNode, attribute))
                else:
                    return_list.append((connectedNode, attribute))


            return return_list

        else:
            # cmds.error('the object %s has no connections %s on the attribute %s' % (self.name, conn_type, in_attribute) )
            # if self.VERBOSE: cmds.warning('the object %s has no connections %s on the attribute %s' % (self.name, conn_type, in_attribute) )
            return None

    def connect_attribute(self, in_attribute, object_to_connect, attribute_to_connect, force = False):
        '''
        Connect two nodes attributes

        Args:
            in_attribute: The attribute to connect from this node
            object_to_connect: The other node as hmObjectNode
            attribute_to_connect: The attribute on the other object
            force: if True the already created connection will be disconnected in favour of the new one

        Returns:
            This is a description of what is returned.

        Raises:
            error: Raises an exception.
        '''
        cmds.connectAttr('{}.{}'.format(self.name, in_attribute),
                         '{}.{}'.format(object_to_connect.get_name(), attribute_to_connect),
                         force = force)

    def break_connection(self, in_attribute, conn_type = None):
        '''
        Break the connection on the specified attribute.

        Args:
            in_attribute: The attribute to break connection to.
            conn_type: The side of breaking if specified. Valid Types are 'in' or 'out'

        Returns:
            This is a description of what is returned.

        Raises:
            error: Raises an exception.
        '''


        # Query both if not specified
        in_conn = self.get_connected(in_attribute, 'in')
        out_conn = self.get_connected(in_attribute, 'out')

        if conn_type == 'in':
            # Break only the in
            out_conn = None
            # print 'trovo', in_conn
        elif conn_type == 'out':
            # Break only the out
            in_conn = None
            # print 'trovo', out_conn


        # Careful, the direction of breaking is DIFFERENT between the two directions!
        if in_conn:
            # For every connected node
            for inConnection in in_conn:
                cmds.disconnectAttr('{}.{}'.format(inConnection[0].get_name(), inConnection[1]),
                                    '{}.{}'.format(self.name, in_attribute))

                # Print message
                if self.VERBOSE: print('break inConnection for',\
                                '{}.{}'.format(inConnection[0].get_name(), inConnection[1]),\
                                '{}.{}'.format(self.name, in_attribute))

        if out_conn:
            # For every connected node
            for outConnection in out_conn:
                cmds.disconnectAttr('{}.{}'.format(self.name, in_attribute),
                                    '{}.{}'.format(outConnection[0].get_name(), outConnection[1]))

                # Print message
                if self.VERBOSE: print('break inConnection for', \
                                 '{}.{}'.format(self.name, in_attribute),\
                                 '{}.{}'.format(outConnection[0].get_name(), outConnection[1]))


        if not in_conn and not out_conn:
            cmds.error('No connections to break.')


    # ================================================================================================================ #
    #================================================= ATTRIBUTE METHODS ============================================= #
    # ================================================================================================================ #
    def has_attribute(self, in_attribute):
        '''
        Write here what the function does.

        Args:
            in_attribute: The attribute to query

        Returns:
            Bool: True is has attribute, False if not
        '''

        return cmds.attributeQuery(in_attribute, node = self.name, exists = True)


    def set_attribute(self, in_attribute, value):
        '''
        Write here what the function does.

        Args:
            in_attribute: The attribute to set
            value: The value of the attribute

        Returns:
            HmBaseObject of the connected attribute
        '''
        # print (self.name + '.' + in_attribute, value)
        return cmds.setAttr('{}.{}'.format(self.name, in_attribute), value)

    def get_attribute(self, in_attribute):
        '''
        Write here what the function does.

        Args:
            in_attribute: The attribute to query

        Returns:
            This is a description of what is returned.

        Raises:
            error: Raises an exception.
        '''
        return cmds.getAttr('{}.{}'.format(self.name, in_attribute))

    def add_attribute(self, attr_name, **kwargs):
        cmds.addAttr(self.name, ln=attr_name, **kwargs)

    # ================================================================================================================ #
    #==================================================== SHAPE METHODS ============================================== #
    # ================================================================================================================ #
    def get_type(self):
        '''
        Get the type of initialized object

        Args:
            None

        Returns:
            The string rapresenting the maya node type.
        '''
        return cmds.nodeType( self.name )

    def is_type(self, input_type):
        '''
        Check if the current object is of the specified input_type

        Args:
            input_type: The type of object to verify the belonging

        Returns:
            True if the current object is of the specified type, False if not
        '''
        node_type = self.get_type()

        if node_type == input_type:
            return True

        return False

    def get_shape(self, all_shapes = False):
        '''
        Get the shape or shapes of the current node

        Args:
            param1: This is the first param.

        Returns:
            HmBaseObject of the shape

        Raises:
            error: Raises an exception if object doesn't exist
        '''
        # Get the shapes of the object
        shapes_data = cmds.listRelatives( self.name, fullPath=True, shapes = True )

        # print 'trovate', shapes_data

        # If there is a related shape
        if shapes_data:
            # Return the first or all the found shapes
            if all_shapes:
                # print 'ritorno una lista con tante belle shapes'
                return_list = [ HmBaseObject( shapes_data[x] ) for x in range(len(shapes_data)) ]
                return return_list
            else:
                # print 'ritorno solo la prima shape trovata'
                return HmBaseObject( shapes_data[0] )
        else:
            if self.VERBOSE: cmds.warning('the object %s has no related shapes' % self.name )
            # print 'non ritorno un cazzo'
            return None

    def get_transform(self):
        '''
        Get the transform of the current node

        Args:
            None

        Returns:
            Return an HmBaseObject for the transform or the parent object of tehe current node, if something is found.is
            Return None if nothing is found

        Raises:
            error: Raises a warning if is already the transform node or if no transform can be found.
        '''
        parent_data = cmds.listRelatives(self.name, p=True, f=True)

        # If there is a related parent
        if parent_data:
            return HmBaseObject( parent_data[0] )
        else:
            if self.VERBOSE: cmds.warning('the object %s has no related transforms or parent' % self.name )
            return None

    # ================================================================================================================ #
    #============================================ HARD MESH SPECIFIC METHODS ========================================= #
    # ================================================================================================================ #
    def reset_display_overrides(self):
        '''
        Reset the display overrides used by Hard Mesh on both the transform and shape

        Args:
            None

        Returns:
            None
        '''
        # Remove any visibility override on the transform
        self.set_attribute('overrideEnabled', 0)

        if self.get_connected('overrideVisibility', 'in'):
            self.break_connection('overrideVisibility')

        self.set_attribute('overrideVisibility', 1)
        # And on the shape
        self.get_shape().set_attribute('overrideEnabled', 0)
        self.get_shape().set_attribute('overrideShading', 1)


    def remove_hm_attributes(self):
        '''
        Remove any kind of special attribute added from Hard Mesh.
        Used for cleaning the input meshes after a operation delete, or a duplicate.

        Args:
            None

        Returns:
            None
        '''
        # Get the current custom attributes
        customAttrs = cmds.listAttr( self.name, userDefined=1 )

        if customAttrs:
            # Remove identification attributes
            for id_attr in self.OBJECT_TYPES:
                if id_attr in customAttrs:
                    cmds.deleteAttr(self.name, at= id_attr )

            # Remove hmDivision attributes
            if nc.smooth_attr_name in customAttrs:
                cmds.deleteAttr(self.name, at= nc.smooth_attr_name )
            # Remove refMeshVis attributes
            if nc.sub_ref_attr_name in customAttrs:
                cmds.deleteAttr(self.name, at= nc.sub_ref_attr_name )

    def get_object_type(self):
        '''
        Return the object type between the valid declared types: hm_in_mesh, hm_out_mesh, hm_inter_curve


        Args:
            None

        Returns:
            The object type as String or None
        '''
        # Get the current custom attributes
        customAttrs = cmds.listAttr( self.name, userDefined=1 )

        # If the output mesh attribute is found, add it to the output mesh list
        if customAttrs:
            # In mesh
            if self.OBJECT_TYPES[0] in customAttrs:
                # Print message
                if self.VERBOSE: print('%s is an %s' % (self.name, self.OBJECT_TYPES[0]))

                return self.OBJECT_TYPES[0]
            # Out mesh
            elif self.OBJECT_TYPES[1] in customAttrs:
                # Print message
                if self.VERBOSE: print('%s is an %s' % (self.name, self.OBJECT_TYPES[1]))

                return self.OBJECT_TYPES[1]
            # Curve
            elif self.OBJECT_TYPES[2] in customAttrs:
                # Print message
                if self.VERBOSE: print('%s is an %s' % (self.name, self.OBJECT_TYPES[2]))

                return self.OBJECT_TYPES[2]

            else:
                # Print message
                if self.VERBOSE: print('%s is not an Hard Mesh Object.' % self.name)

                return None


