#!/usr/bin/env python
# -*- coding: utf-8 -*-

import maya.cmds as cmds

# naming conventions
import hmScripts.hm_global_variables as nc
import hmScripts.Main.hmBaseObject as hmNode



class HmSelectionHandling():
    '''
    This class implements some common methods for handling and validating selections.


    EXAMPLE USAGE:
    #Init the sel handling
    selection = hmSelect.HmSelectionHandling()

    #Set custom messages
    selection.setErrorMessageIfZero('Please select 2 meshes to operate on.')
    selection.setErrorIfSmallThanMin('Please select 2 meshes to operate on.')
    selection.setErrorMessageIfOutLimit('Please select only 2 objects. You can have multiple shells for each mesh.')

    #Validate the selection and put the result on a var
    sel = selection.validate(2,2,'mesh')


    '''
    def __init__(self):
        #===== Prendiamo la selezione..
        self.initSel = cmds.ls( sl=True, long=True, fl = True)

        #Validation parameters
        self.min = None
        self.max = None
        self.type = None
        self.extractType = None

        #Error messages
        self.errorIfZero = None
        self.errorIfSmallThanMin = None
        self.errorIfOutLimit =None
    #
    #
    # def get_current(self, clean_paths):
    #


    def setFromList(self, inList):
        '''
        Set the selection to operate on from a list instead.
        '''
        self.initSel = inList


    def extract(self, selType):
        '''
        This commands extract the specified elements the list if present.
        If nothing is found nothing happens.
        This is useful for passing this extracted elements to a validate in a different moment.

        currently supported types:
        mesh
        nurbsCurve
        '''

        # Here we save the extracted elements
        extractedList = []
        objToKeep = []

        for obj in self.initSel:

            # double check the transform and the shape for being sure to extract the right type
            shape = cmds.listRelatives( obj, fullPath=True, shapes = True )
            # check the object
            if cmds.objectType( obj, isType= selType ):
                extractedList.append(obj)
            # check the shape
            elif shape is not None:
                # print shape, selType
                if cmds.objectType( shape, isType= selType ):
                    extractedList.append( obj )
            # if nothing match, leave it to the main list
            else:
                objToKeep.append( obj )

        # Save the object to keep in the class list
        self.initSel = objToKeep

        # return the found objects of matching type
        return extractedList

    def ignoreComponents(self):
        '''
        Ignore component selection by getting only the hilited objects
        '''
        componentSelection = self.initSel

        self.initSel = cmds.ls(hl=True) #
        # self.initSel = cmds.ls( objectsOnly=True )

        return componentSelection


    def validate(self, selMin=None, selMax=None, selType=None, allow_zero_sel = False):
        '''
        selMin: the minimum valid items, if None is ignored. when components are selected this is the number of objects
        selMax: the maximum valid items, if None is ignored. when components are selected this is the number of objects
        selType: the type of element valid for passing the test, if None is ignored
        allow_zero_sel: if true let the validation pass even with an empty selection. Used in some context.

        currently supported types:
        mesh
        hm_in_mesh (maya mesh with the hidden attribute)


        edges
        '''

        #Set current data
        self.min = selMin
        self.max = selMax
        self.type = selType


        # if self.type == 'edges' or 'vertices' or 'faces':
        # 	# Save here the supposed components
        # 	self.componentSelection = self.initSel
        # 	# store the objects
        # 	self.initSel = cmds.ls(hl=True)

        # If the zero selection is not allowed
        if not allow_zero_sel:
            #===== Validiamo la selezione
            if len(self.initSel) == 0:
                if self.errorIfZero is None:
                    cmds.error( 'Selection is empty' )
                else:
                    cmds.error( self.errorIfZero )


        #Min checking, if required.
        if self.min is not None:
            #If the selection is smaller
            if len(self.initSel) < self.min:
                #Display default or custom error
                if self.errorIfSmallThanMin is None:
                    cmds.error( 'Selection is smaller than %s' % self.min )
                else:
                    cmds.error( self.errorIfSmallThanMin )


        #Max checking, if required
        if self.max is not None:
            #If the selection is bigger
            if len(self.initSel) > self.max:
                #Display default or custom error
                if self.errorIfOutLimit is None:
                    cmds.error( 'Selection is bigger than %s' % self.max )
                else:
                    cmds.error( self.errorIfOutLimit )


        # ================= TYPE CHECKING ===========================
        #If no errors on selections, validate the type EXTEND THIS PART WITH ALL OR ANYWAY MORE SHAPES TYPES
        if self.type == 'mesh':
            for obj in self.initSel:

                shape = cmds.listRelatives( obj, fullPath=True, shapes = True ) [0]
                if cmds.objectType( shape, isType= self.type ) == 0:
                    cmds.error( '%s invalid. This command works only with mesh' % shape )


        if self.type == nc.hm_in_mesh:

            # Cicle throug all the elements in selection
            for obj in self.initSel:
                mesh_obj = hmNode.HmBaseObject(obj)
                hm_type = mesh_obj.get_object_type()

                if hm_type != nc.hm_in_mesh:
                    cmds.error( '%s invalid. This command works only with %s mesh types' % (obj, nc.hm_in_mesh) )

        # if self.type == 'edge':
        # 	edges = cmds.filterExpand(self.componentSelection, sm=31)
        # 	print edges
        # 	if edges == None:
        # 		cmds.error( 'This command works only with poligons edges.' )
        # 	else:
        # 		self.initSel = edges



        return self.initSel



    def setErrorMessageIfZero(self, stringError):
        '''
        Set the error to display if the selection is empty
        '''
        self.errorIfZero = stringError


    def setErrorIfSmallThanMin(self, stringError):
        '''
        Set the error to display if the selection is smaller than minimum
        '''
        self.errorIfSmallThanMin = stringError


    def setErrorMessageIfOutLimit(self, stringError):
        '''
        Set the error to display if the selection is bigger than maximum
        '''
        self.errorIfOutLimit = stringError
















